@@ -3,6 +3,7 @@
 
 import cPickle as pickle
 
+import logging
 import sys
 import os
 import os.path
@@ -14,7 +15,7 @@ import javalang
 import compiler
 import util
 
-def find_source_files(rootpath, excludes):
+def find_source_files(input_path, excludes):
     """ Get a list of filenames for all Java source files within the given
     directory.
 
@@ -22,9 +23,9 @@ def find_source_files(rootpath, excludes):
 
     java_files = []
 
-    rootpath = os.path.normpath(os.path.abspath(rootpath))
+    input_path = os.path.normpath(os.path.abspath(input_path))
 
-    for dirpath, dirnames, filenames in os.walk(rootpath):
+    for dirpath, dirnames, filenames in os.walk(input_path):
         if is_excluded(dirpath, excludes):
             del dirnames[:]
             continue
@@ -137,6 +138,14 @@ def get_newer(a, b):
 
     return a
 
+def format_syntax_error(e):
+    rest = ""
+    if e.at.position:
+        value = e.at.value
+        pos = e.at.position
+        rest = ' at %s line %d, character %d' % (value, pos[0], pos[1])
+    return e.description + rest
+
 def generate_from_source_file(doc_compiler, source_file, cache_dir):
     if cache_dir:
         cache_file = os.path.join(cache_dir, source_file.replace(os.sep, ':')) + '-CACHE'
@@ -152,15 +161,15 @@ def generate_from_source_file(doc_compiler, source_file, cache_dir):
 
     try:
         ast = javalang.parse.parse(source)
+    except javalang.parser.JavaSyntaxError, e:
+        util.error('Syntax error in %s: %s', source_file, format_syntax_error(e))
     except Exception:
-        sys.stderr.write('Exception while parsing ' + source_file + '\n')
-        raise
+        util.unexpected('Unexpected exception while parsing %s', source_file)
 
     try:
         documents = doc_compiler.compile(ast)
     except Exception:
-        sys.stderr.write('Exception while compiling ' + source_file + '\n')
-        raise
+        util.unexpected('Unexpected exception while compiling %s', source_file)
 
     if cache_file:
         dump_file = open(cache_file, 'w')
@@ -210,6 +219,8 @@ def is_excluded(root, excludes):
     return False
 
 def main(argv=sys.argv):
+    logging.basicConfig(level=logging.WARN)
+
     parser = OptionParser(
         usage="""\
 usage: %prog [options] -o <output_path> <input_path> [exclude_paths, ...]
@@ -236,6 +247,8 @@ Note: By default this script will not overwrite already created files.""")
                       help='Don\'t create a table of contents file')
     parser.add_option('-s', '--suffix', action='store', dest='suffix',
                       help='file suffix (default: rst)', default='rst')
+    parser.add_option('-I', '--include', action='append', dest='includes',
+                      help='Additional input paths to scan', default=[])
     parser.add_option('-v', '--verbose', action='store_true', dest='verbose',
                       help='verbose output')
 
@@ -246,15 +259,19 @@ Note: By default this script will not overwrite already created files.""")
 
     rootpath, excludes = args[0], args[1:]
 
+    input_paths = opts.includes
+    input_paths.append(rootpath)
+
     if not opts.destdir:
         parser.error('An output directory is required.')
 
     if opts.suffix.startswith('.'):
         opts.suffix = opts.suffix[1:]
 
-    if not os.path.isdir(rootpath):
-        sys.stderr.write('%s is not a directory.\n' % (rootpath,))
-        sys.exit(1)
+    for input_path in input_paths:
+        if not os.path.isdir(input_path):
+            sys.stderr.write('%s is not a directory.\n' % (input_path,))
+            sys.exit(1)
 
     if not os.path.isdir(opts.destdir):
         os.makedirs(opts.destdir)
@@ -263,7 +280,10 @@ Note: By default this script will not overwrite already created files.""")
         os.makedirs(opts.cache_dir)
 
     excludes = normalize_excludes(rootpath, excludes)
-    source_files = find_source_files(rootpath, excludes)
+    source_files = []
+
+    for input_path in input_paths:
+        source_files.extend(find_source_files(input_path, excludes))
 
     packages, documents, sources = generate_documents(source_files, opts.cache_dir, opts.verbose)
 
